﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using DTS.Common.Collections;
using DTS.Common.PerformanceData;
using DTS.Common.DataAccess.Services;

namespace DTS.Common.DataAccess.Services.Example
{
    public partial class PdbQueryClientForm : Form
    {
        public PdbQueryClientForm()
        {
            InitializeComponent();
        }

        // Filter variables
        public bool useInstanceFilters = false;
        public bool useCounterFilters = false;
        public PerformanceObjectInstanceFilter instanceFilter = null;
        public PerformanceCounterThresholdTestFilter counterFilter = null;
        private DTS.Common.DataAccess.Services.PerformanceDataService service = null;

        // Get the collected objects on a machine
        private void GetCollectedObjects(string machine)
        {
            Machine mo = new Machine();
            mo.MachineName = machine;
            GetCollectedObjects(mo);
        }

        // Get the collected objects on a machine with the Machine object signature
        private void GetCollectedObjects(Machine machine)
        {
            PerformanceObjects Objects = null;
            try
            {
                Objects = service.GetCollectionObjects(machine);
            }
            catch (Exception e2)
            {
                MessageBox.Show("Exception occurred while getting the collection object names.\r\n\r\n" + e2.Message, "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            // Clear the list box and put these strings in the list box
            this.lbCollectedObjects.Items.Clear();
            this.cbObjectNames.Items.Clear();

            if (Objects == null)
            {
                MessageBox.Show("Could not get the collection object from the machine " + machine.MachineName + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                foreach (PerformanceObject po in Objects)
                {
                    this.lbCollectedObjects.Items.Add(po);
                    this.cbObjectNames.Items.Add(po.Name);
                }
            }

            // Select the first object for convenience
            if (this.cbObjectNames.Items.Count > 0)
                this.cbObjectNames.SelectedIndex = 0;
        }

        private void ClientExForm_Load(object sender, EventArgs e)
        {
            // Create the service object.  The connection will be opened when the dataaccess is performed
            string connectionString = Properties.Settings.Default.ConnectionString; 
            service = new DTS.Common.DataAccess.Services.PerformanceDataService(connectionString);

            // Set the default time for the time controls
            this.EndTime.Value = DateTime.Now;
            this.StartTime.Value = DateTime.Now - new TimeSpan(0, 30, 0);         

            // Set up the chart types
            string[] names = Enum.GetNames(typeof(SeriesChartType));
            Array values = Enum.GetValues(typeof(SeriesChartType));

            // Only use these 3 chart types Line, Area, StackedArea, Bar
            for (int countElements = 0; countElements <= names.Length - 1; countElements++)
            {
                if ((SeriesChartType.Area == (SeriesChartType) values.GetValue(countElements)) ||
                    (SeriesChartType.Line == (SeriesChartType) values.GetValue(countElements)) ||
                    (SeriesChartType.Column ==(SeriesChartType) values.GetValue(countElements)) ||
                    (SeriesChartType.StackedColumn == (SeriesChartType)values.GetValue(countElements)))
                {
                    this.cbChartType.Items.Add( names[countElements].ToString() );
                }
            }

            // Select the first index
            int StackedColumnIndex = this.cbChartType.Items.IndexOf( "StackedColumn" );
            if (StackedColumnIndex == -1)
                this.cbChartType.SelectedIndex = 0;
            else
                this.cbChartType.SelectedIndex = StackedColumnIndex;

            this.DataChart.GetToolTipText += new System.EventHandler<System.Windows.Forms.DataVisualization.Charting.ToolTipEventArgs>(this.DataChart_GetToolTipText);

            this.DataChart.ChartAreas[0].AxisX.LabelStyle.Format = "G";
            this.DataChart.ChartAreas[0].AxisX.Interval = 1;

            RefreshMachines();
            
            EnableControls();
        }

        public SeriesChartType GetSeriesChartType( string Name )
        {
             // Set up the chart types
            string[] names = Enum.GetNames(typeof(SeriesChartType));
            Array values = Enum.GetValues(typeof(SeriesChartType));
            for (int countElements = 0; countElements <= names.Length - 1; countElements++)
            {
                if ( names[countElements] ==  Name )
                    return (SeriesChartType) values.GetValue(countElements);
            }

            return SeriesChartType.Column;
        }

        private void lbCollectedObjects_SelectedValueChanged(object sender, EventArgs e)
        {
            // Get the counters for the object
            // Make sure machine is selected
            string Machine = this.cbMachine.Text;
            if ((Machine != null) && (Machine.Length > 0))
            {
                // Get the object name
                PerformanceObject Object = this.lbCollectedObjects.SelectedItem as PerformanceObject;
                if ((Object != null) && (Object.Name.Length > 0))
                {
                    PerformanceCounters counters = null;

                    try
                    {
                        counters = service.GetCounters(Machine, Object);
                    }
                    catch (Exception e3)
                    {
                        System.Console.WriteLine(e3.Message);
                    }

                    if ((counters == null) || (counters.Count == 0))
                    {
                        MessageBox.Show("Could not get counter names of the object " + Object.Name);
                    }
                    else
                    {
                        this.lbCounters.Items.Clear();
                        foreach (PerformanceCounter pc in counters)
                        {
                            this.lbCounters.Items.Add(pc);
                        }
                        Object.Counters = counters;
                    }                      
                }
            }

            EnableControls();
        }

        private void lbCounters_SelectedValueChanged(object sender, EventArgs e)
        {
            EnableControls();

            RefreshData();
        }
        
        
        public void RefreshData()
        {
            // Get the data for this counter from the selected object on the selected machine
            string Machine = this.cbMachine.Text;
            if ((Machine != null) && (Machine.Length < 1))
                return;

            PerformanceObject Object = this.lbCollectedObjects.SelectedItem as PerformanceObject;
            if ((Object == null) || (Object.Name.Length < 1))
                return;

            PerformanceCounter Counter = this.lbCounters.SelectedItem as PerformanceCounter;
            if ((Counter == null) || (Counter.Name.Length < 1))
                return;
           
            PerformanceDataRequest request = new PerformanceDataRequest();
            try
            {
                // Set up the performance object
                PerformanceObject po = new PerformanceObject(Object.Name);

                // Prepare the data request
                request = prepareDataRequest(po, Counter);

                PerformanceDataSamples dataSamples = new PerformanceDataSamples();

                // Clear out the items
                this.lvValues.Items.Clear();
                // Clear the titles including the YAxis
                for (int i = 0; i < DataChart.Titles.Count; i++)
                    DataChart.Titles[i].Text = "";
                
                // List of performance data on each machine requested
                List<DTS.Common.PerformanceData.PerformanceData> result = service.GetPerformanceData(request);
                if ((result != null) && (result.Count > 0))
                {                    
                    // Transform the data into performance data samples for the requested object and counter
                    result[0].ToSamples(po,
                        Counter.Name,
                        dataSamples);

                    // Adds Titles if there is data returned
                    if (dataSamples.Count != 0)
                    {
                        DataChart.Titles["TitleLine1"].Text = Object + "/" + Counter + " on " + Machine;
                        DataChart.Titles["TitleLine2"].Text = "From " + StartTime.Value + " to " + EndTime.Value;
                        DataChart.Titles["YAxis"].Text = Counter.Name;
                    }

                    // Add the new items & data to the chart
                    IEnumerator e = this.DataChart.Series.GetEnumerator();
                    while (e.MoveNext())
                    {
                        Series s = (Series)e.Current;
                        s.Points.Clear();
                    }

                    this.DataChart.Series.Clear();
                      
                    Hashtable indexedInstances = new Hashtable();

                    // There is one data sample for each instance at a particular time
                    for (int i = 0; i < dataSamples.Count; i++)
                    {
                        PerformanceDataSample sample = dataSamples[i] as PerformanceDataSample;

                        // Don't plot ID Process for Process Object if we specified another counter
                        if ((po.Name == "Process") 
                            && (po.Counters.Count > 1)
                            && (sample.CounterName == "ID Process"))
                            continue;

                        // Define the list items
                        ListViewItem lvi = new ListViewItem(sample.CollectionTime.ToString());
                        lvi.SubItems.Add(sample.UniqueInstanceName);
                        lvi.SubItems.Add(sample.Value);

                        // Add the list items to the ListView
                        this.lvValues.Items.Add(lvi);

                        // Add one series to the chart for each instance of the object requested
                        Series s = null;

                        // use 'uninstanced' for the instance name
                        string InstanceName = "";
                        if (sample.UniqueInstanceName.Length == 0)
                            InstanceName = "Uninstanced";
                        else
                            InstanceName = sample.UniqueInstanceName;

                        s = this.DataChart.Series.FindByName(InstanceName);
                        if (s == null)
                        {
                            s = new Series(InstanceName);
                            s.XValueType = ChartValueType.DateTime;
                            this.DataChart.Series.Add(s);
                        }
                            
                        // Set the ChartType
                        if (cbChartType.SelectedItem == null)
                        {
                            s.ChartType = SeriesChartType.Column;
                        }
                        else
                        {
                            s.ChartType = GetSeriesChartType(cbChartType.SelectedItem.ToString());
                        }

                        // Add the point CollectionTime,Value
                        if ((s != null) && (s.Points != null))
                        {
                            int iAdded = s.Points.AddXY(sample.CollectionTime, sample.DValue);
                            if (iAdded >= 0)
                            {
                                DataPoint dp = s.Points[iAdded];
                                dp["TTlabel"] = sample.CollectionTime.ToShortTimeString();
                            }
                        }
 
                    }

                    if (this.lvValues.Items.Count == 0)
                    {
                        MessageBox.Show("No data was found for the specified time period.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            }
            catch (Exception e3)
            {
                MessageBox.Show("Could not get performance counter data " + e3.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                System.Console.WriteLine(e3.Message);
            }
            finally
            {
            }
        }

        private PerformanceDataRequest prepareDataRequest(PerformanceObject po, PerformanceCounter counter)
        {
            PerformanceDataRequest request = new PerformanceDataRequest();
            
            // Set up the performance counter
            if (!po.Counters.Contains(counter))
                po.Counters.Add(counter);

            // Create the performance object request list
            request.ObjectsRequested = new PerformanceObjects();
            request.ObjectsRequested.Add(po);            
            request.Machines = new string[] { cbMachine.Text };

            request.StartTime = this.StartTime.Value;
            request.EndTime = this.EndTime.Value;

            if (this.cbIncludeOnlyTopX.Checked == true)
            {
                try
                {
                    double TopXValue = Convert.ToDouble(this.topXUpDown.Value);
                    request.PerformanceCounterTopXFilters = new PerformanceCounterTopXFilters();
                    request.PerformanceCounterTopXFilters.Add(new PerformanceCounterTopXFilter(po.Name, 
                        counter.Name, 
                        TopXValue));
                }
                catch (Exception e)
                {
                    MessageBox.Show("The TopX filter value must be numeric.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            // use instance filter
            if ((this.cbUseInstanceFilter.Checked == true) && (this.instanceFilter != null))
            {
                request.InstanceFilters = new PerformanceObjectInstanceFilters();
                request.InstanceFilters.Add(this.instanceFilter);
            }

            setupCounterFilters(po, counter, request);

            // Set up the join to filter
            if ((this.cbIdProcess.Enabled == true) 
                && (this.cbIdProcess.Checked == true)
                && (this.cbObjectNames.SelectedIndex != -1))
            {
                // Add the IdProcess join to the request
                // Find the process object
                PerformanceObjectJoinToFilter filter = new PerformanceObjectJoinToFilter();
                filter.JoinToCounterName = "ID Process";
                filter.JoinToObjectName = this.cbObjectNames.Text;
                filter.ObjectName = po.SQLCompliantName;

                request.JoinToFilters = new PerformanceObjectJoinToFilters();
                request.JoinToFilters.Add(filter);
            }

            return request;
        }

        private void setupCounterFilters(PerformanceObject Object, PerformanceCounter Counter, PerformanceDataRequest request)
        {
            // use counter filter
            PerformanceCounterThresholdTestType Type = PerformanceCounterThresholdTestType.GT;
            if (this.cbUseCounterFilter.Checked == true)
            {
                request.PerformanceCounterTestFilters = new PerformanceCounterThresholdTestFilters();
                switch (cbCounterFilterOperator.Text)
                {
                    case ">":
                        Type = PerformanceCounterThresholdTestType.GT;
                        break;

                    case ">=":
                        Type = PerformanceCounterThresholdTestType.GTE;
                        break;
                    case "<=":
                        Type = PerformanceCounterThresholdTestType.LTE;
                        break;
                    case "<":
                        Type = PerformanceCounterThresholdTestType.LT;
                        break;
                    case "=":
                        Type = PerformanceCounterThresholdTestType.EQ;
                        break;
                    default:
                        MessageBox.Show("Select counter filter type when the use counter checkbox is checked.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;
                }

                double ThresholdValue = 0.0;
                try
                {
                    ThresholdValue = double.Parse(tbCounterFilterValue.Text);
                }
                catch (Exception e)
                {
                    System.Console.WriteLine(e.Message);
                    MessageBox.Show("Counter filter value must be numeric.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                PerformanceCounterThresholdTestFilter perfCounterTestFilter = new PerformanceCounterThresholdTestFilter(Object.Name, Counter.Name, ThresholdValue, Type);
                request.PerformanceCounterTestFilters.Add(perfCounterTestFilter);

            }
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            RefreshData();
        }

        private void btnUseCurrentEndTime_Click(object sender, EventArgs e)
        {
            this.EndTime.Value = System.DateTime.Now;
        }

        private void btnUseCurrentStartTime_Click(object sender, EventArgs e)
        {
            this.StartTime.Value = System.DateTime.Now;
        }

        private void cbChartType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbChartType.SelectedItem == null)
                return;
            if (this.DataChart.Series == null)
                return;

            // Add the new items & data to the chart
            IEnumerator e2 = this.DataChart.Series.GetEnumerator();
            while (e2.MoveNext())
            {
                Series s = (Series)e2.Current;
                s.ChartType = GetSeriesChartType(cbChartType.SelectedItem.ToString());
            }
        }

        private void EnableControls()
        {
            this.btnFilters.Enabled = this.cbUseInstanceFilter.Checked 
                && (this.lbCollectedObjects.SelectedItem != null);

            this.StartTime.Enabled = true;
            this.EndTime.Enabled = true;
            this.btnUseCurrentEndTime.Enabled = true;
            this.cbObjectNames.Enabled = ( this.lbCollectedObjects.Text == "Process" );
            this.cbIdProcess.Enabled = (this.lbCollectedObjects.Text == "Process");
        }

        private void btnFilters_Click(object sender, EventArgs e)
        {
            // Make sure the filter button is enabled for this
            if (this.lbCollectedObjects.SelectedItem == null)
                return;

            // launch the filter dialog
            FilterDialog f = new FilterDialog();
            f.instanceFilter = this.instanceFilter;
            f.CounterName = this.lbCounters.SelectedItem as string;
            PerformanceObject po = this.lbCollectedObjects.SelectedItem as PerformanceObject;
            if (po != null)
                f.ObjectName = po.Name;
            
            DialogResult result = f.ShowDialog();
            if (result == DialogResult.OK)
            {
                this.instanceFilter = f.instanceFilter;
                if (this.cbUseInstanceFilter.Checked)
                    RefreshData();
            }
        }

        private void cbChartType_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            RefreshData();
        }
        
        private void cbUseCounterFilter_CheckedChanged(object sender, EventArgs e)
        {
            RefreshData();
        }

        private void cbUseInstanceFilter_CheckedChanged(object sender, EventArgs e)
        {
            EnableControls();

            RefreshData();
        }

        private void btnRefresh_Click_1(object sender, EventArgs e)
        {
            RefreshData();
        }

        private void DataChart_GetToolTipText(object sender, System.Windows.Forms.DataVisualization.Charting.ToolTipEventArgs e)
        {
            // Check selevted chart element and set tooltip
            if (e.HitTestResult.ChartElementType == ChartElementType.DataPoint)
            {
                int i = e.HitTestResult.PointIndex;
                DataPoint dp = e.HitTestResult.Series.Points[i];
                string Units = DataChart.Titles["YAxis"].Text;
                if (e.HitTestResult.Series.Name == "Uninstanced")
                {
                    e.Text = string.Format("{0}\r\n{1:F1} {2}", dp["TTlabel"], dp.YValues[0], Units);
                }
                else
                {
                    e.Text = string.Format("{0}\r\nInstance {1}={2:F1} {3}",
                        dp["TTlabel"],
                        e.HitTestResult.Series.Name,
                        dp.YValues[0],
                        Units);
                }
            }
        }

        private void btnRefreshMachine_Click(object sender, EventArgs e)
        {
            RefreshMachines();
        }

        private void RefreshMachines()
        {
            try
            {
                List<Machine> machines = service.GetAllMachines();
                this.cbMachine.Items.Clear();
                foreach (Machine m in machines)
                {
                    this.cbMachine.Items.Add(m.MachineName);
                }
                if (cbMachine.Items.Count > 0)
                    cbMachine.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                
            }
        }

        private void cbMachine_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbMachine.SelectedIndex != -1)
            {
                Machine machine = new Machine();
                machine.MachineName = cbMachine.Text;
                this.GetCollectedObjects(machine);
            }
        }
    }
}
